/*
 *  Copyright (C) 2013  Thadeu Lima de Souza Cascardo <cascardo@cascardo.info>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

#include "friend.h"
#include <string.h>
#include <glib.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <errno.h>
#include <stdio.h>
#include "message.h"

struct friend {
	char *name;
	char *address;
	uint16_t port;
	GInetSocketAddress *saddr;
};

static GSocket *usock;

int sock_init(void)
{
	GSocketAddress *address;
	GInetAddress *any_addr;
	any_addr = g_inet_address_new_any(G_SOCKET_FAMILY_IPV6);
	usock = g_socket_new(G_SOCKET_FAMILY_IPV6, G_SOCKET_TYPE_DATAGRAM, G_SOCKET_PROTOCOL_UDP, NULL);
	address = g_inet_socket_address_new(any_addr, 17078);
	g_socket_bind(usock, address, TRUE, NULL);
	g_object_unref(any_addr);
	g_object_unref(address);
	message_init(usock);
	return 0;
}

int friend_send_message(struct friend *friend, char *buffer, size_t len)
{
	g_socket_send_to(usock, G_SOCKET_ADDRESS(friend->saddr), buffer, len, NULL, NULL);
	return 0;
}

struct cache {
	GList *friends;
};

static struct cache *ucache;

struct friend *friend_get_by_address(GInetAddress *address)
{
	GList *l;
	for (l = g_list_first(ucache->friends); l != NULL; l = g_list_next(l)) {
		struct friend *friend = l->data;
		if (g_inet_address_equal(g_inet_socket_address_get_address(friend->saddr), address))
			return friend;
	}
	return NULL;
}

int create_cache(struct cache **cache)
{
	ucache = *cache = g_slice_new0(struct cache);
	(*cache)->friends = NULL;
	return 0;
}

static void destroy_friend(gpointer data)
{
	struct friend *friend = data;
	g_free(friend->name);
	g_free(friend->address);
	g_object_unref(friend->saddr);
	g_slice_free(struct friend, friend);
}

int destroy_cache(struct cache *cache)
{
	if (cache->friends)
		g_list_free_full(cache->friends, destroy_friend);
	g_slice_free(struct cache, cache);
}

char * friend_get_name(struct friend *friend)
{
	return friend->name;
}

int cache_add_friend(struct cache *cache, char *name, char *address, uint16_t port)
{
	struct friend *friend;
	GInetAddress *addr;
	friend = g_slice_new0(struct friend);
	friend->name = g_strdup(name);
	friend->address = g_strdup(address);
	friend->port = port;
	addr = g_inet_address_new_from_string(address);
	friend->saddr = G_INET_SOCKET_ADDRESS(g_inet_socket_address_new(addr, friend->port));
	g_object_unref(addr);
	cache->friends = g_list_append(cache->friends, friend);
	return 0;
}

int load_cache(struct cache *cache, char *fname)
{
	GKeyFile *file;
	gchar **groups;
	gchar **group;
	file = g_key_file_new();
	g_key_file_load_from_file(file, fname, G_KEY_FILE_KEEP_COMMENTS | G_KEY_FILE_KEEP_TRANSLATIONS, NULL);
	groups = g_key_file_get_groups(file, NULL);
	for (group = groups; *group != NULL; group++) {
		gchar *name;
		gchar *address;
		uint16_t port;
		name = g_key_file_get_value(file, *group, "name", NULL);
		address = g_key_file_get_value(file, *group, "address", NULL);
		port = g_key_file_get_integer(file, *group, "port", NULL);
		cache_add_friend(cache, name, address, port);
		g_free(name);
		g_free(address);
	}
	g_strfreev(groups);
	g_key_file_free(file);
	return 0;
}

int store_cache(struct cache *cache, char *fname)
{
	GKeyFile *file;
	GList *f;
	gchar *contents;
	gssize len;
	file = g_key_file_new();
	g_key_file_load_from_file(file, fname, G_KEY_FILE_KEEP_COMMENTS | G_KEY_FILE_KEEP_TRANSLATIONS, NULL);
	for (f = g_list_first(cache->friends); f != NULL; f = g_list_next(f)) {
		struct friend *friend = f->data;
		g_key_file_set_value(file, friend->name, "name", friend->name);
		g_key_file_set_value(file, friend->name, "address", friend->address);
		g_key_file_set_integer(file, friend->name, "port", friend->port);
	}
	contents = g_key_file_to_data(file, &len, NULL);
	g_file_set_contents(fname, contents, len, NULL);
	g_free(contents);
	g_key_file_free(file);
	return 0;
}
